//interfaceoptions.cpp
//user interface for setting game options

#include <OpenGL/gl.h>
#include "gametime.h"
#include <string.h>
#include <stdio.h>
#include "interfaceutil.h"
#include "gamemem.h"
#include "fileio.h"
#include "screen.h"
#include "controls.h"
#include "text.h"
#include "interface.h"
#include "gameinitexit.h"
#include "parser.h"
#include "writeout.h"
#include "textures.h"
#include "config.h"
#include "gamesound.h"
#include "textures.h"
#include "renderframe.h"
#include "music.h"

#define kNumControlOptions 16

void InterfaceControlOptions()
{
	int controlOptions[kNumControlOptions]={kInputGasButton,kInputBrakeButton,kInputSteerLeftButton,kInputSteerRightButton,kInputHandbrakeButton,kInputKickdownButton,kInputCamera,kInputCameraReverse,kInputCameraChangeCar,kInputPause,kInputGearUp,kInputGearDown,kInputHorn,kInputChat,kInputITunesNext,kInputITunesPlay};

	tInterfaceMenuDescribtion menu;
	InterfaceInitMenu(&menu,kNumControlOptions+1,"Controls");

	strcpy(menu.items[kNumControlOptions].label,"Return to previous menu");
	menu.items[kNumControlOptions-1].lineSpacing*=1.5;
	menu.background=FileGetReference("background-options.tif");
	menu.RenderCallback=InterfaceRenderReplay;
	menu.itemsYPos+=0.1;
	
	char itemStrings[kNumControlOptions][32]={"Accelerate","Brake","Steer Left","Steer Right","Handbrake","Full Throttle","Change Camera","Look Backwards","Replay Car View","Pause Game","Shift Up","Shift Down","Horn","Chat ","iTunes Skip","iTunes Play"};
	for(int i=0;i<kNumControlOptions;i++)
	{
		menu.items[i].size*=0.7;
		menu.items[i].lineSpacing*=0.6;
	}
		
	int exit=false;
	do{
		for(int i=0;i<kNumControlOptions;i++)
			if(gConfig->keys[controlOptions[i]].controllerID1)
				sprintf(menu.items[i].label,"%s:\255#a\255\t%s\t%s",itemStrings[i],gConfig->keys[controlOptions[i]].identifier,gConfig->keys[controlOptions[i]].controllerIdentifier);		
			else
				sprintf(menu.items[i].label,"%s:\255#a\255\t%s",itemStrings[i],gConfig->keys[controlOptions[i]].identifier);		
					
		InterfaceMenuZoomAnimation(&menu,-1,true);
		int sel=InterfaceGetUserMenuSelection(&menu);
		if(sel!=kInterfaceMenuEsc&&sel<kNumControlOptions)
		{
			while(GetInterfaceKey(kInterfaceKeyReturn)||GetInterfaceKey(kInterfaceKeyEnter));

			menu.initialSelection=sel;
			char keyString[80];
			sprintf(keyString,"Press the new Key or Button for '%s'\n (Or press delete to clear)",itemStrings[sel]);
			InterfaceDrawStrings("Assign new Key",keyString,kFileErr);

			GetInput(gConfig->keys+controlOptions[sel]);
			
			int interfaceKeyPressed;
			do{
				interfaceKeyPressed=false;
				for(int i=kInterfaceKeyUp;i<kInterfaceNumKeys;i++)
					if(GetInterfaceKey(i))
						interfaceKeyPressed=true;
				if(GetButtonInput(kInputITunesNext)||GetButtonInput(kInputITunesPlay))
					interfaceKeyPressed=true;
			}while(interfaceKeyPressed);
		} 
		else
			exit=true;
		
	}while(!exit);

	InterfaceDisposeMenu(&menu);
}

#define kNumTauntOptions 10
void InterfaceTauntOptions()
{
	tInterfaceMenuDescribtion menu;
	InterfaceInitMenu(&menu,kNumTauntOptions+1,"Taunts");

	strcpy(menu.items[kNumTauntOptions].label,"Return to previous menu");
	menu.items[kNumTauntOptions-1].lineSpacing*=1.5;
	menu.background=FileGetReference("background-options.tif");
	menu.RenderCallback=InterfaceRenderReplay;
	
	char itemStrings[5][32];
	for(int i=0;i<kNumTauntOptions;i++)
	{
		menu.items[i].size*=0.75;
		if(i%2)
		{
			menu.items[i].lineSpacing*=0.9;
			menu.items[i].flags|=kInterfaceMenuItemTypeable;
			strcpy(menu.items[i].type,gConfig->taunts[i/2]);
			sprintf(menu.items[i].label,"Taunt %d message: ",i/2+1);
		}
		else
		{
			menu.items[i].lineSpacing*=0.65;
			sprintf(itemStrings[i/2],"Taunt %d key",i/2+1);
		}
	}
		
	int exit=false;
	do{
		for(int i=0;i<kNumTauntOptions;i++)
			if(!(i%2))
			{
				if(gConfig->keys[kInputTaunt1+i/2].controllerID1)
					sprintf(menu.items[i].label,"%s:\255#a\255\t%s\t%s",itemStrings[i/2],gConfig->keys[kInputTaunt1+i/2].identifier,gConfig->keys[kInputTaunt1+i/2].controllerIdentifier);		
				else
					sprintf(menu.items[i].label,"%s:\255#a\255\t%s",itemStrings[i/2],gConfig->keys[kInputTaunt1+i/2].identifier);		
			}
					
		InterfaceMenuZoomAnimation(&menu,-1,true);
		int sel=InterfaceGetUserMenuSelection(&menu);
		if(sel!=kInterfaceMenuEsc&&sel<kNumTauntOptions&&!(sel%2))
		{
			while(GetInterfaceKey(kInterfaceKeyReturn)||GetInterfaceKey(kInterfaceKeyEnter));

			menu.initialSelection=sel;
			char keyString[80];
			sprintf(keyString,"Press the new Key or Button for '%s'",itemStrings[sel/2]);
			InterfaceDrawStrings("Assign new Key",keyString,kFileErr);

			GetInput(gConfig->keys+kInputTaunt1+sel/2);
			
			int interfaceKeyPressed;
			do{
				interfaceKeyPressed=false;
				for(int i=kInterfaceKeyUp;i<kInterfaceNumKeys;i++)
					if(GetInterfaceKey(i))
						interfaceKeyPressed=true;
			}while(interfaceKeyPressed);
		} 
		else
			exit=true;
		
	}while(!exit);

	for(int i=1;i<kNumTauntOptions;i+=2)
		strcpy(gConfig->taunts[i/2],menu.items[i].type);

	InterfaceDisposeMenu(&menu);
}

void InterfaceCalibrateAxis()
{
	if(gConfig->axis[kInputSteerAxis].axisControllerID1)
	{
		InterfaceDrawStrings("Calibrate Axis","Move the Steering Axis to it's left and right \nmaxium several times and hit Return.",kFileErr);
		CalibrateAxis(kInputSteerAxis);
	}
	if(gConfig->axis[kInputThrottleBrakeAxis].axisControllerID1)
	{
		InterfaceDrawStrings("Calibrate Axis","Move the Gas/Brake Axis to it's left and right \nmaxium several times and hit Return.",kFileErr);
		CalibrateAxis(kInputThrottleBrakeAxis);
	}
	FlushKeys();
}

enum{
	kAnalougeSteerAxis,
	kAnalougeGasAxis,
	kAnalougeCalibrate,
	kAnalougeFFB,
	kAnalougeFFBIntensity,
	kAnalougeExit,
	kAnalougeNumOptions
};

void InterfaceAnalogueControlOptions()
{
	int analogueControlOptions[]={kInputSteerAxis,kInputThrottleBrakeAxis};

	tInterfaceMenuDescribtion menu;
	InterfaceInitMenu(&menu,kAnalougeNumOptions,"Analouge Controls");
	menu.background=FileGetReference("background-options.tif");
	menu.RenderCallback=InterfaceRenderReplay;

	
	char itemStrings[][32]={"Steering","Gas/Brake","Calibrate Axis...","","FFB Intensity:","Return to previous menu"};

	for(int i=0;i<kAnalougeNumOptions;i++)
		sprintf(menu.items[i].label,"%s",itemStrings[i]);		
	
	menu.items[kAnalougeFFB].flags|=kInterfaceMenuItemArrowInput;
	menu.items[kAnalougeFFBIntensity].flags|=kInterfaceMenuItemArrowInput|kInterfaceMenuItemSlider;
	if(!gFFB)
	{
		menu.items[kAnalougeFFB].flags|=kInterfaceMenuItemDisabled;
		menu.items[kAnalougeFFBIntensity].flags|=kInterfaceMenuItemDisabled;
	}
	if(!gInputHID)
	{
		menu.items[kAnalougeSteerAxis].flags|=kInterfaceMenuItemDisabled;
		menu.items[kAnalougeGasAxis].flags|=kInterfaceMenuItemDisabled;
		menu.items[kAnalougeCalibrate].flags|=kInterfaceMenuItemDisabled;
	}
	menu.items[kAnalougeExit-1].lineSpacing*=1.5;

	for(int i=0;i<kAnalougeExit;i++)
	{
		menu.items[i].size*=0.75;
		menu.items[i].lineSpacing*=0.75;
	}

	int exit=false;
	do{
		for(int i=0;i<=kAnalougeGasAxis;i++)
			if(gConfig->axis[analogueControlOptions[i]].axisControllerID1)
				sprintf(menu.items[i].label,"%s: \255#a\255%s",itemStrings[i],gConfig->axis[analogueControlOptions[i]].axisIdentifier);		
			else
				sprintf(menu.items[i].label,"%s: \255#a\255not assigned",itemStrings[i]);		
				
		sprintf(menu.items[kAnalougeFFB].label,"Force Feedback: \255#a\255%s",gConfig->ffb?"On":"Off");
		menu.items[kAnalougeFFBIntensity].sliderPos=gConfig->ffbIntensity;
		if(gInputHID&&(gConfig->axis[kInputThrottleBrakeAxis].axisControllerID1||gConfig->axis[kInputSteerAxis].axisControllerID1))
			menu.items[kAnalougeCalibrate].flags&=~kInterfaceMenuItemDisabled;
		else
			menu.items[kAnalougeCalibrate].flags|=kInterfaceMenuItemDisabled;			

		int sel=InterfaceGetUserMenuSelection(&menu)&kInterfaceMenuItemMask;
		menu.initialSelection=sel;
		if(sel<=kAnalougeGasAxis&&sel!=kInterfaceMenuEsc)
		{
			while(GetInterfaceKey(kInterfaceKeyReturn)||GetInterfaceKey(kInterfaceKeyEnter));

			menu.initialSelection=sel;
			char keyString[80];
			sprintf(keyString,"Move the new axis for '%s'",itemStrings[sel]);
			InterfaceDrawStrings("Assign new Axis",keyString,kFileErr);

			GetAxis(gConfig->axis+analogueControlOptions[sel]);
			InterfaceMenuZoomAnimation(&menu,-1,true);
		} 
		else if(sel==kAnalougeCalibrate)
		{
			InterfaceCalibrateAxis();
			InterfaceMenuZoomAnimation(&menu,-1,true);
		}
		else if(sel==kAnalougeFFB)
			gConfig->ffb=!gConfig->ffb;
		else if(sel==kAnalougeFFBIntensity)
		{
			if(sel&kInterfaceMenuLeftArrow){
				gConfig->ffbIntensity-=0.05;
				if(gConfig->ffbIntensity<0)gConfig->ffbIntensity=0;
			}
			else{
				gConfig->ffbIntensity+=0.05;
				if(gConfig->ffbIntensity>1.0)gConfig->ffbIntensity=1.0;
			}
		}
		else exit=true;
		
	}while(!exit);
	InterfaceDisposeMenu(&menu);
}

enum{
	kVideoOptionScreenSize,
	kVideoOptionFullscreen,
	kVideoOptionShadows,
	kVideoOptionTextureFilter,
	kVideoOptionTextureQuality,
	kVideoOptionClipDistance,
	kVideoOptionInteriorDisplay,
	kVideoOptionColor32Bit,
	kVideoOptionFSAA,
	kVideoOptionReturn,
	kNumVideoOptions
};

#define kMaxTextureQuality 9
#define kMinTextureQuality -1

void InterfaceVideoOptions()
{
	int screenReInitRequired=false;
	int textureReLoadRequired=false;
	int glReInitRequired=false;
	
	int videoMode;
	for(int i=0;i<gVideoNumModes;i++)
		if(gConfig->screenXSize==gVideoModes[i].width&&gConfig->screenYSize==gVideoModes[i].height)
			videoMode=i;
	printf("%d/%d\n",videoMode,gVideoNumModes);
	tInterfaceMenuDescribtion menu;
	InterfaceInitMenu(&menu,kNumVideoOptions,"Video Options");
	menu.background=FileGetReference("background-options.tif");
	menu.RenderCallback=InterfaceRenderReplay;

	char itemStrings[][256]={"Set the display resolution for Redline to use. Larger values means sharper graphics, but may slow the game down.",
		"Switch between fullscreen and windowed modes. You can also switch at any time by pressing Command-F.",
		"Turn shadows and light cones on or off. High quality shadows may slow down the game quite a lot.",
		"Changes texture filter. Anisotropic filtering results in the smoothest graphics, Bilinear filtering is the fastest.",
		"Changes the texture quality. Higher quality means sharper graphics, but may cause the game to run too slow, if your graphics card doesn't have enough memory.",
		"Changes the visible display range. Higher values let you see farther, but will cause the game to run slower.",
		"Turns car interor display on or off. Turning interior display off will let the game run faster.",
		"Changes between 16-bit and 32-bit color modes. 32-bit looks much nicer, but may be slower on older graphics cards.",
		"Turns on Full screen Anti-Aliasing. Will cause the game to look smoother, but may cause slowdowns. \255#r\255Requires you to restart Redline to take effect.",
		""
		};
	for(int i=0;i<menu.numItems;i++)
		strcpy(menu.items[i].describtion,itemStrings[i]);


	strcpy(menu.items[kVideoOptionReturn].label,"Return to previous menu");
	menu.items[kVideoOptionReturn-1].lineSpacing*=1.5;
	
	for(int i=kVideoOptionScreenSize;i<=kVideoOptionFSAA;i++)
		menu.items[i].flags|=kInterfaceMenuItemArrowInput;

	menu.items[kVideoOptionClipDistance].flags|=kInterfaceMenuItemSlider;
	strcpy(menu.items[kVideoOptionClipDistance].label,"Clipping Distance:");
	menu.items[kVideoOptionTextureQuality].flags|=kInterfaceMenuItemSlider;
	strcpy(menu.items[kVideoOptionTextureQuality].label,"Texture Quality:");
	menu.enterAlwaysOK=true;
	for(int i=0;i<kVideoOptionReturn;i++)
	{
		menu.items[i].size*=0.75;
		menu.items[i].lineSpacing*=0.75;
	}

	int exit=false;
	int zoom=false;
	do{
		sprintf(menu.items[kVideoOptionScreenSize].label,"Screen Size: \255#a\255%dx%d",gVideoModes[videoMode].width,gVideoModes[videoMode].height);
		sprintf(menu.items[kVideoOptionFullscreen].label,"Screen Mode: \255#a\255%s",gConfig->fullscreen?"Fullscreen":"Window");
		switch(gConfig->textureFilter){
			case 0: strcpy(menu.items[kVideoOptionTextureFilter].label,"Texture Filter: \255#a\255Bilinear");break;
			case 1: strcpy(menu.items[kVideoOptionTextureFilter].label,"Texture Filter: \255#a\255Trilinear");break;
			case 2: strcpy(menu.items[kVideoOptionTextureFilter].label,"Texture Filter: \255#a\255Anisotropic");break;
		}
		switch(gConfig->fsaa){
			case 0: strcpy(menu.items[kVideoOptionFSAA].label,"Anti-Aliasing: \255#a\255Off");break;
			case 2: strcpy(menu.items[kVideoOptionFSAA].label,"Anti-Aliasing: \255#a\255Medium");break;
			case 4: strcpy(menu.items[kVideoOptionFSAA].label,"Anti-Aliasing: \255#a\255Full");break;
		}
		switch(gConfig->stencil){
			case 0: strcpy(menu.items[kVideoOptionShadows].label,"Shadows & Lighting: \255#a\255Off");break;
			case 1: strcpy(menu.items[kVideoOptionShadows].label,"Shadows & Lighting: \255#a\255Normal");break;
			case 3: strcpy(menu.items[kVideoOptionShadows].label,"Shadows & Lighting: \255#a\255Good (Slow)");break;
			case 6: strcpy(menu.items[kVideoOptionShadows].label,"Shadows & Lighting: \255#a\255Really Good (Really Slow)");break;
			case 12: strcpy(menu.items[kVideoOptionShadows].label,"Shadows & Lighting: \255#a\255Insanely Good (Insanely Slow)");break;
		}
		switch(gConfig->interiorDisplay){
			case kInteriorDisplayOff: strcpy(menu.items[kVideoOptionInteriorDisplay].label,"Interior Display: \255#a\255Off");break;
			case kInteriorDisplayPlayerOnly: strcpy(menu.items[kVideoOptionInteriorDisplay].label,"Interior Display: \255#a\255Player Only");break;
			case kInteriorDisplayCloseCarsOnly: strcpy(menu.items[kVideoOptionInteriorDisplay].label,"Interior Display: \255#a\255Nearby Cars Only");break;
			case kInteriorDisplayAlways: strcpy(menu.items[kVideoOptionInteriorDisplay].label,"Interior Display: \255#a\255Always");break;
		}
		menu.items[kVideoOptionClipDistance].sliderPos=(gConfig->clipFarDistance-150)/450.0;
		menu.items[kVideoOptionTextureQuality].sliderPos=1-((float)gConfig->textureQuality-kMinTextureQuality)/(kMaxTextureQuality-kMinTextureQuality);
		sprintf(menu.items[kVideoOptionColor32Bit].label,"Color Depth: \255#a\255%d",gConfig->color32Bit?32:16);
		
		if(!zoom){
			InterfaceMenuZoomAnimation(&menu,-1,true);
			zoom=true;
		}

		int sel=InterfaceGetUserMenuSelection(&menu);
		switch(menu.initialSelection=(sel&kInterfaceMenuItemMask))
		{
			case kVideoOptionScreenSize:
				if(sel&kInterfaceMenuLeftArrow)
					videoMode=(videoMode+1)%gVideoNumModes;
				else{
					videoMode--;
					if(videoMode<0)videoMode=gVideoNumModes-1;
				}
				screenReInitRequired=true;
				break;
			case kVideoOptionFullscreen:
				gConfig->fullscreen=!gConfig->fullscreen;
				screenReInitRequired=true;
				break;
			case kVideoOptionShadows:
				if(!(sel&kInterfaceMenuLeftArrow))
					switch(gConfig->stencil){
						case 0: gConfig->stencil=1;break;
						case 1: gConfig->stencil=3;break;
						case 3: gConfig->stencil=0;break;
					//	case 6: gConfig->stencil=0;break;
					}
				else
					switch(gConfig->stencil){
						case 0: gConfig->stencil=3;break;
						case 1: gConfig->stencil=0;break;
						case 3: gConfig->stencil=1;break;
					//	case 6: gConfig->stencil=3;break;
					}
			//	screenReInitRequired=true;
				break;
			case kVideoOptionTextureQuality:
				if(!(sel&kInterfaceMenuLeftArrow)){
					if(gConfig->textureQuality>kMinTextureQuality)
						gConfig->textureQuality--;
				}
				else{
					if(gConfig->textureQuality<kMaxTextureQuality)
						gConfig->textureQuality++;
				}
				textureReLoadRequired=true;
				break;
			case kVideoOptionTextureFilter:
				if(!(sel&kInterfaceMenuLeftArrow)){
					gConfig->textureFilter++;
					if(gConfig->textureFilter>2)
						gConfig->textureFilter=0;
				}				
				else{
					gConfig->textureFilter--;
					if(gConfig->textureFilter<0)
						gConfig->textureFilter=2;
				}
				textureReLoadRequired=true;
				break;
			case kVideoOptionClipDistance:
				if(sel&kInterfaceMenuLeftArrow){
					if(gConfig->clipFarDistance>150)
					gConfig->clipFarDistance-=10;
				}
				else{
					if(gConfig->clipFarDistance<600)
					gConfig->clipFarDistance+=10;
				}
				glReInitRequired=true;
				break;
			case kVideoOptionInteriorDisplay:
				if(sel&kInterfaceMenuLeftArrow){
					gConfig->interiorDisplay--;
					if(gConfig->interiorDisplay<0)gConfig->interiorDisplay=kInteriorDisplayNumModes-1;
				}
				else
					gConfig->interiorDisplay=(gConfig->interiorDisplay+1)%kInteriorDisplayNumModes;				
				break;
			case kVideoOptionColor32Bit:
				gConfig->color32Bit=!gConfig->color32Bit;
				screenReInitRequired=true;
				textureReLoadRequired=true;
				break;
			case kVideoOptionFSAA:
				if(sel&kInterfaceMenuLeftArrow){
					gConfig->fsaa-=2;
					if(gConfig->fsaa<0)gConfig->fsaa=4;
				}
				else
					gConfig->fsaa=(gConfig->fsaa+2)%6;				
				screenReInitRequired=true;
				break;

			case kVideoOptionReturn:
			case kInterfaceMenuEsc:
			case kInterfaceMenuOK:
				exit=true;
				break;
		}
	}while(!exit);
	
	gConfig->screenXSize=gVideoModes[videoMode].width;
	gConfig->screenYSize=gVideoModes[videoMode].height;
	
	if(textureReLoadRequired)
		TexturesUnloadAll();
	if(screenReInitRequired)
		ScreenReInit();
	else
	{
		if(glReInitRequired)
			InitGL();
	}
	InterfaceDisposeMenu(&menu);
}

enum{
	kGameplayOptionsMetric,
	kGameplayOptionsAutomatic,
	kGameplayOptionsCamera,
	kGameplayOptionsTransparency,
	kGameplayOptionsName,
	kGameplayOptionsRegisterLapTimes,
	kGameplayOptionsShowPlayerNames,
	kGameplayOptionsCornerGuides,
	kGameplayOptionsReturn,
	kNumGameplayOptionsButtons
};

void InterfaceGameplayOptions()
{
	tInterfaceMenuDescribtion menu;
	InterfaceInitMenu(&menu,kNumGameplayOptionsButtons,"Gameplay Options");

	char itemStrings[][256]={"Switches the game to use either metric or imperial units for speed, power, mass, etc.",
		"Switches between using an automatic or manual transmission.",
		"Switches the default camera mode.",
		"Changes the transparency of in-game displays.",
		"Lets you change your name used in network games and the internet track records.",
		"If you enable this option, your lap times in Time Trial and Challenge modes will be submitted to the internet track record database.",
		"If you enable this, player names will be displayed above the cars in network games.",
		"Show guiding signs informing you about upcoming corners.",
		""
		};
	for(int i=0;i<menu.numItems;i++)
		strcpy(menu.items[i].describtion,itemStrings[i]);

	strcpy(menu.items[kGameplayOptionsReturn].label,"Return to previous menu");
	menu.items[kGameplayOptionsReturn-1].lineSpacing*=1.5;
	
	menu.items[kGameplayOptionsMetric].flags|=kInterfaceMenuItemArrowInput;
	menu.items[kGameplayOptionsAutomatic].flags|=kInterfaceMenuItemArrowInput;
	menu.items[kGameplayOptionsCamera].flags|=kInterfaceMenuItemArrowInput;
	menu.items[kGameplayOptionsRegisterLapTimes].flags|=kInterfaceMenuItemArrowInput;
	menu.items[kGameplayOptionsShowPlayerNames].flags|=kInterfaceMenuItemArrowInput;
	menu.items[kGameplayOptionsCornerGuides].flags|=kInterfaceMenuItemArrowInput;
	menu.items[kGameplayOptionsTransparency].flags|=kInterfaceMenuItemArrowInput|kInterfaceMenuItemSlider;
	strcpy(menu.items[kGameplayOptionsTransparency].label,"HUD Transparency:");
	menu.items[kGameplayOptionsName].flags|=kInterfaceMenuItemTypeable;
	menu.items[kGameplayOptionsName].maxTypeLength=kMaxNameLength-1;
	
	strcpy(menu.items[kGameplayOptionsName].label,"Network Name: ");
	strcpy(menu.items[kGameplayOptionsName].type,gConfig->playerName);

	menu.background=FileGetReference("background-options.tif");
	menu.RenderCallback=InterfaceRenderReplay;

	menu.enterAlwaysOK=true;
	
	for(int i=0;i<kGameplayOptionsReturn;i++)
	{
		menu.items[i].size*=0.75;
		menu.items[i].lineSpacing*=0.75;
	}

	int exit=false;
	int zoom=false;
	do{
		sprintf(menu.items[kGameplayOptionsMetric].label,"Units: \255#a\255%s",gConfig->metricUnits?"Metric":"Imperial");
		sprintf(menu.items[kGameplayOptionsAutomatic].label,"Transmission: \255#a\255%s",gConfig->automatic?"Automatic":"Manual");
		sprintf(menu.items[kGameplayOptionsRegisterLapTimes].label,"Internet Lap Time Registering: \255#a\255%s",gConfig->registerLapTimes?"On":"Off");
		sprintf(menu.items[kGameplayOptionsCornerGuides].label,"Show Corner Guides: \255#a\255%s",gConfig->guideSigns?"On":"Off");
		sprintf(menu.items[kGameplayOptionsShowPlayerNames].label,"Show Network Player Names: \255#a\255%s",gConfig->showPlayerNames?"On":"Off");
		switch(gConfig->cameraMode)
		{
			case kCameraChase:
				sprintf(menu.items[kGameplayOptionsCamera].label,"Default Camera: \255#a\255Chase");break;
			case kCameraChaseClose:
				sprintf(menu.items[kGameplayOptionsCamera].label,"Default Camera: \255#a\255Close Chase");break;
			case kCameraCockpit:
				sprintf(menu.items[kGameplayOptionsCamera].label,"Default Camera: \255#a\255Cockpit");break;
			case kCameraCockpitCarHidden:
				sprintf(menu.items[kGameplayOptionsCamera].label,"Default Camera: \255#a\255Cockpit");break;
		}
		menu.items[kGameplayOptionsTransparency].sliderPos=gConfig->hudTransparency*1/0.8;
		menu.items[kGameplayOptionsTransparency].sliderTransparency=gConfig->hudTransparency;
		
		if(!zoom){
			InterfaceMenuZoomAnimation(&menu,-1,true);
			zoom=true;
		}

		int sel=InterfaceGetUserMenuSelection(&menu);
		switch(menu.initialSelection=(sel&kInterfaceMenuItemMask))
		{
			case kGameplayOptionsMetric:
				gConfig->metricUnits=!gConfig->metricUnits;
				break;

			case kGameplayOptionsRegisterLapTimes:
				gConfig->registerLapTimes=!gConfig->registerLapTimes;
				break;

			case kGameplayOptionsCornerGuides:
				gConfig->guideSigns=!gConfig->guideSigns;
				break;

			case kGameplayOptionsAutomatic:
				gConfig->automatic=!gConfig->automatic;
				break;
				
			case kGameplayOptionsCamera:
				if(sel&kInterfaceMenuLeftArrow){
					gConfig->cameraMode--;
					if(gConfig->cameraMode<kCameraChase)gConfig->cameraMode=kCameraCockpitCarHidden;
				}
				else{
					gConfig->cameraMode++;
					if(gConfig->cameraMode>kCameraCockpitCarHidden)gConfig->cameraMode=kCameraChase;
				}
				break;

			case kGameplayOptionsTransparency:
				if(sel&kInterfaceMenuLeftArrow){
					gConfig->hudTransparency-=0.05;
					if(gConfig->hudTransparency<0)gConfig->hudTransparency=0;
				}
				else{
					gConfig->hudTransparency+=0.05;
					if(gConfig->hudTransparency>0.8)gConfig->hudTransparency=0.8;
				}
				break;
				
			case kGameplayOptionsShowPlayerNames:
				gConfig->showPlayerNames=!gConfig->showPlayerNames;
				break;
				
			case kGameplayOptionsReturn:
			case kInterfaceMenuEsc:
			case kInterfaceMenuOK:
				exit=true;
				break;
		}
	}while(!exit);

	strcpy(gConfig->playerName,menu.items[kGameplayOptionsName].type);

	InterfaceDisposeMenu(&menu);
}

enum{
	kAudioOptionsVolume,
	kAudioOptionsMusicVolume,
	kAudioOptionsReturn,
	kNumAudioOptionsButtons
};

void InterfaceAudioOptions()
{
	tInterfaceMenuDescribtion menu;
	InterfaceInitMenu(&menu,kNumAudioOptionsButtons,"Audio Options");

	strcpy(menu.items[kAudioOptionsReturn].label,"Return to previous menu");
	menu.items[kAudioOptionsReturn-1].lineSpacing*=1.5;
	
	strcpy(menu.items[kAudioOptionsVolume].label,"Volume:");
	menu.items[kAudioOptionsVolume].flags|=kInterfaceMenuItemArrowInput|kInterfaceMenuItemSlider;

	strcpy(menu.items[kAudioOptionsMusicVolume].label,"Music Volume:");
	menu.items[kAudioOptionsMusicVolume].flags|=kInterfaceMenuItemArrowInput|kInterfaceMenuItemSlider;

	int exit=false;
	menu.enterAlwaysOK=true;

	for(int i=0;i<kAudioOptionsReturn;i++)
	{
		menu.items[i].size*=0.75;
		menu.items[i].lineSpacing*=0.75;
	}
	
	menu.background=FileGetReference("background-options.tif");
	menu.RenderCallback=InterfaceRenderReplay;

	InterfaceMenuZoomAnimation(&menu,-1,true);
	do{
		menu.items[kAudioOptionsVolume].sliderPos=gConfig->soundVolume;
		menu.items[kAudioOptionsMusicVolume].sliderPos=gConfig->musicVolume;
			
		int sel=InterfaceGetUserMenuSelection(&menu);
		switch(menu.initialSelection=(sel&kInterfaceMenuItemMask))
		{
			case kAudioOptionsVolume:
				if(sel&kInterfaceMenuLeftArrow){
					gConfig->soundVolume-=0.1;
					if(gConfig->soundVolume<0)gConfig->soundVolume=0;
				}
				else{
					gConfig->soundVolume+=0.1;
					if(gConfig->soundVolume>1)gConfig->soundVolume=1;
				}
				SoundReInit();
				break;

			case kAudioOptionsMusicVolume:
				if(sel&kInterfaceMenuLeftArrow){
					gConfig->musicVolume-=0.1;
					if(gConfig->musicVolume<0)gConfig->musicVolume=0;
				}
				else{
					gConfig->musicVolume+=0.1;
					if(gConfig->musicVolume>1)gConfig->musicVolume=1;
				}
				MusicSetVolume();
				break;
								
			case kAudioOptionsReturn:
			case kInterfaceMenuEsc:
			case kInterfaceMenuOK:
				exit=true;
				break;
		}
	}while(!exit);
	
	InterfaceDisposeMenu(&menu);
}


enum{
	kOptionsGameplay,
	kOptionsControls,
	kOptionsAnalougeControls,
	kOptionsTaunts,
	kOptionsVideo,
	kOptionsAudio,
	kOptionsReturn,
	kNumOptionsButtons
};

void InterfaceOptions()
{
	tInterfaceMenuDescribtion menu;
	InterfaceInitMenu(&menu,kNumOptionsButtons,"Options");

	char itemStrings[][32]={"Gameplay","Controls","Analogue Controls","Multiplayer Taunts","Video","Audio","Return to previous menu"};

	for(int i=0;i<menu.numItems;i++)
	{
		strcpy(menu.items[i].label,itemStrings[i]);
		menu.items[i].size*=1.15;
		menu.items[i].lineSpacing*=1.15;
	}

	menu.items[kOptionsReturn-1].lineSpacing*=1.5;

	menu.background=FileGetReference("background-options.tif");
	menu.RenderCallback=InterfaceRenderReplay;
	
	int exit=false;
	do{
		InterfaceMenuZoomAnimation(&menu,-1,true);
		switch(menu.initialSelection=InterfaceGetUserMenuSelection(&menu))
		{
			case kOptionsGameplay:
				InterfaceGameplayOptions();
				break;
			case kOptionsVideo:
				InterfaceVideoOptions();
				break;
			case kOptionsControls:
				InterfaceControlOptions();
				break;
			case kOptionsAnalougeControls:
				InterfaceAnalogueControlOptions();
				break;
			case kOptionsTaunts:
				InterfaceTauntOptions();
				break;
			case kOptionsAudio:
				InterfaceAudioOptions();
				break;
				
			case kOptionsReturn:
			case kInterfaceMenuEsc:
				exit=true;
				break;
		}
	}while(!exit);
	InterfaceDisposeMenu(&menu);
	WriteOutFile(FileGetReference("config.cfg"),gConfig,kParserTypeConfigDesc);
}
